import argparse

from dotenvhub import __version__

from .config import cfg
from .constants import SHELLS


def parse_args(args):
    """Parse command line parameters

    Args:
      args (List[str]): command line parameters as list of strings
          (for example  ``["--help"]``).

    Returns:
      :obj:`argparse.Namespace`: command line parameters namespace
    """

    parser = argparse.ArgumentParser(description="DotEnvHub your .env file manager")
    subparsers = parser.add_subparsers(dest="mode")
    parser.add_argument(
        "--version",
        action="version",
        version=f"kanban-python {__version__}",
    )
    sub_exp = subparsers.add_parser("copy", help="Export target File to CWD")
    sub_exp.add_argument(
        "filename",
        help="Export target File to CWD",
    )
    sub_exp.add_argument(
        "-N",
        "--name",
        help="Name of file to create in CWD",
        default=".env",
    )

    sub_copy = subparsers.add_parser(
        "shell", help="Shell to generate export string for"
    )
    sub_copy.add_argument(
        "filename",
        help="Filename where string is stored in DotEnvHub like FOLDER/FILE",
    )
    sub_copy.add_argument(
        "-S",
        "--shell",
        help=f"""
        Shell to generate Export String for
        (default: last selected shell in dotenvhub)
        (current:{cfg.shell})
        """,
        choices=SHELLS,
        default=cfg.shell,
    )

    return parser.parse_args(args)



import sys
from pathlib import Path

import pytest

from dotenvhub import utils


@pytest.mark.skipif(sys.platform == "linux", reason="does not run on ubuntu")
def test_copy_path_to_clipboard():
    test_path = "test_folder/test_file"
    copied_path = utils.copy_path_to_clipboard(path=Path(test_path))
    assert test_path == copied_path


@pytest.mark.skipif(sys.platform == "linux", reason="does not run on ubuntu")
@pytest.mark.parametrize(
    "shell, expected_shell_str ",
    [
        ("cmd", 'set "USER=TESTUSER" & set "DB=TESTDB" & set "PORT=TESTPORT"'),
        ("pwsh", '$env:USER="TESTUSER" ; $env:DB="TESTDB" ; $env:PORT="TESTPORT"'),
        ("bash", "export USER=TESTUSER ; export DB=TESTDB ; export PORT=TESTPORT"),
        ("zsh", "export USER=TESTUSER ; export DB=TESTDB ; export PORT=TESTPORT"),
    ],
)
def test_create_shell_export_str(shell, test_file_content, expected_shell_str):
    copied_str = utils.create_shell_export_str(
        shell=shell, env_content=test_file_content
    )

    assert copied_str == expected_shell_str



from pathlib import Path

from textual import on
from textual.app import ComposeResult
from textual.containers import Vertical
from textual.reactive import reactive
from textual.screen import ModalScreen
from textual.validation import Regex
from textual.widgets import Button, Input, Label, TextArea

from ..config import cfg
from ..constants import ENV_FILE_DIR_PATH, SHELLS
from ..utils import update_file_tree, write_to_file
from .filepanel import EnvFileSelector


class ModalShellSelector(ModalScreen):
    CSS_PATH = Path("../assets/modal_shell.css")

    def compose(self) -> ComposeResult:
        shell_buttons = [
            Button(label=shell, id=shell, variant="primary", classes="shell")
            for shell in SHELLS
        ]
        for btn in shell_buttons:
            btn.can_focus = False
        yield Vertical(
            Label("Which Shell are you using?"),
            *shell_buttons,
            Button("Cancel", id="btn-modal-cancel"),
            id="modal-shell-vert",
        )

    @on(Button.Pressed, "#btn-modal-cancel")
    def close_window(self) -> None:
        self.dismiss()

    @on(Button.Pressed, ".shell")
    def pop_up_shell_select(self, event: Button.Pressed) -> None:
        self.dismiss()
        selected_shell = event.button.id
        self.app.current_shell = selected_shell
        cfg.shell = self.app.current_shell

        self.app.query_one("#btn-shell-select").label = self.app.current_shell


class ModalSaveScreen(ModalScreen):
    CSS_PATH = Path("../assets/modal_save.css")
    preview = reactive(":page_facing_up: Enter File Name")

    def compose(self) -> ComposeResult:
        yield Vertical(
            Label("How to save file: e.g. FOLDER/FILE"),
            Input(
                placeholder="New File Name",
                id="inp-new-file-name",
                valid_empty=False,
                validators=[Regex("^[a-zA-Z0-9_.]*(/[a-zA-Z0-9_.]*)?$")],
                validate_on=["changed", "submitted"],
            ),
            Label(self.preview, id="lbl-new-file-name"),
            Button("Save", id="btn-modal-save"),
            Button("Cancel", id="btn-modal-cancel"),
            id="modal-save-vert",
        )

    @on(Input.Submitted)
    def press_button(self):
        self.query_one("#btn-modal-save", Button).press()

    @on(Button.Pressed, "#btn-modal-cancel")
    def close_window(self) -> None:
        self.dismiss()
        self.app.query_one(TextArea).disabled = False
        self.app.query_one(TextArea).focus()

    @on(Button.Pressed, "#btn-modal-save")
    def save_new_file(self) -> None:
        self.dismiss()

        new_path = ENV_FILE_DIR_PATH / self.query_one(Input).value
        write_to_file(path=new_path, content=self.app.current_content)

        self.app.file_tree = update_file_tree()

        self.app.query_one(EnvFileSelector).remove()
        self.app.query_one("#app-grid").mount(
            EnvFileSelector(id="file-selector"), before="#file-preview"
        )

    @on(Input.Changed, "#inp-new-file-name")
    def format_name(self, event: Input.Changed):
        text = event.input.value or event.input.placeholder
        if "/" not in text:
            preview_name = f":page_facing_up: {text}"
        elif len(text.split("/")) == 2:
            folder, file = [el.strip() for el in text.split("/")]
            preview_name = f":file_folder: {folder} / :page_facing_up: {file}"
        else:
            preview_name = ":cross_mark: Enter a valid Folder/File Name"

        self.preview = preview_name

        if self.query_one(Input).is_valid:
            self.query_one("#btn-modal-save", Button).disabled = False
        else:
            self.query_one("#btn-modal-save", Button).disabled = True

        self.query_one("#lbl-new-file-name").remove()
        self.mount(
            Label(self.preview, id="lbl-new-file-name"), after="#inp-new-file-name"
        )



from pathlib import Path

from textual import on
from textual.containers import VerticalScroll
from textual.widgets import Button, Collapsible, Label, ListItem, ListView, TextArea

from ..constants import ENV_FILE_DIR_PATH
from ..utils import get_env_content, update_file_tree


class EnvFileSelector(VerticalScroll):
    def compose(self):
        for dirpath, filenames in self.app.file_tree.items():
            if dirpath == ".":
                general_list = ListView(
                    *[
                        ListItem(
                            Label(f":page_facing_up: {file}"),
                            Button.warning(
                                "Edit", id=f"btn-edit-{file}", classes="edit"
                            ),
                            Button.error(
                                "Delete", id=f"btn-del-{file}", classes="delete"
                            ),
                            id=f"file-{file}",
                        )
                        for file in filenames
                    ],
                    initial_index=None,
                )
                yield general_list
            else:
                folder_list = ListView(
                    *[
                        ListItem(
                            Label(f":page_facing_up: {file}"),
                            Button.warning(
                                "Edit", id=f"btn-edit-{dirpath}/{file}", classes="edit"
                            ),
                            Button.error(
                                "Delete",
                                id=f"btn-del-{dirpath}/{file}",
                                classes="delete",
                            ),
                            id=f"{dirpath}-{file}",
                        )
                        for file in filenames
                    ],
                    id=f"collaps-{dirpath}",
                    initial_index=None,
                )

                folder_colabs = Collapsible(
                    folder_list,
                    title=dirpath,
                    collapsed_symbol=":file_folder:",
                    expanded_symbol=":open_file_folder:",
                )
                yield folder_colabs

    @on(ListView.Selected)
    def get_preview_file_path(self, event: ListView.Selected):
        self.app.file_to_show = event.list_view.highlighted_child.id.split("-")[1]
        self.query(Button).remove_class("active")
        event.list_view.highlighted_child.query(Button).add_class("active")

        # only collapsible lists have ID
        if event.list_view.id:
            folder = Path(event.list_view.id.split("-")[1])
            self.app.file_to_show_path = (
                ENV_FILE_DIR_PATH / folder / self.app.file_to_show
            )
            self.app.query_one(
                "#file-preview"
            ).border_title = f"{folder} / {self.app.file_to_show}"
        else:
            self.app.file_to_show_path = ENV_FILE_DIR_PATH / self.app.file_to_show
            self.app.query_one("#file-preview").border_title = self.app.file_to_show

    @on(ListView.Selected)
    def reset_highlights(self, event: ListView.Selected):
        for lviews in self.query(ListView):
            if lviews.id != event.list_view.id:
                lviews.index = None

    @on(ListView.Selected)
    def enable_buttons(self):
        self.app.query_one("#btn-shell-export").disabled = False
        self.app.query_one("#btn-file-export").disabled = False
        self.app.query_one("#btn-copy-path").disabled = False

        self.app.query_one("#btn-new-file").disabled = False
        self.app.query_one("#btn-save-file").disabled = True

    @on(ListView.Selected)
    def update_preview_text(self):
        self.app.current_content = get_env_content(filepath=self.app.file_to_show_path)

        text_widget = self.app.query_one(TextArea)
        text_widget.text = self.app.current_content
        text_widget.action_cursor_page_down()
        text_widget.disabled = True

    @on(Button.Pressed, ".delete")
    def delete_env_file(self, event: Button.Pressed):
        folder_file_path = event.button.id[8:]

        # Delete File
        (ENV_FILE_DIR_PATH / folder_file_path).unlink()
        try:
            # If Folder Empty delete Folder
            (ENV_FILE_DIR_PATH / folder_file_path).parent.rmdir()
        except OSError:
            pass

        self.app.file_tree = update_file_tree()
        self.app.query_one(EnvFileSelector).remove()
        self.app.query_one("#app-grid").mount(
            EnvFileSelector(id="file-selector"), before="#file-preview"
        )

        # Clear Text Missing Border Title
        self.app.file_to_show = ""
        self.app.file_to_show_path = ""
        self.app.current_content = ""

        text_widget = self.app.query_one(TextArea)
        text_widget.text = ""

    @on(Button.Pressed, ".edit")
    def edit_env_file(self):
        text_widget = self.app.query_one(TextArea)
        text_widget.disabled = False
        text_widget.focus()



from textual import on
from textual.containers import Horizontal
from textual.widgets import TextArea


class FilePreviewer(Horizontal):
    def compose(self):
        yield TextArea(id="text-preview")

    @on(TextArea.Changed)
    def disable_buttons(self):
        save_button = self.app.query_one("#btn-save-file")
        save_button.disabled = False



import os
import shutil
from pathlib import Path

import pyperclip
from rich.console import Console

from .constants import ENV_FILE_DIR_PATH

console = Console()


def update_file_tree(path: Path = ENV_FILE_DIR_PATH) -> dict:
    file_tree_dict = {}
    for dirpath, _, filenames in os.walk(path):
        rel_path = Path(dirpath).relative_to(path)
        file_tree_dict[f"{rel_path}"] = filenames

    return file_tree_dict


def copy_path_to_clipboard(path: Path) -> str:
    str_path = path.as_posix()
    pyperclip.copy(str_path)
    return str_path


def get_env_content(filepath: Path):
    try:
        with open(filepath, "r") as env_file:
            return "".join(env_file.readlines())
    except FileNotFoundError:
        console.print("File [red]not found[/], make sure you entered a valid filename")


def create_copy_in_cwd(filename: str, filepath: Path):
    cwd = Path.cwd()
    try:
        shutil.copy(filepath, cwd / filename)
        console.print(f"Created [blue]{filename}[/] in CWD")
    except FileNotFoundError:
        console.print("File [red]not found[/], make sure you entered a valid filename")


def create_shell_export_str(shell, env_content):
    if shell == "pwsh":
        return create_pwsh_string(env_content=env_content)
    if shell == "cmd":
        return create_cmd_string(env_content=env_content)
    if shell == "bash":
        return create_bash_string(env_content=env_content)
    if shell == "zsh":
        return create_bash_string(env_content=env_content)


def create_pwsh_string(env_content: str):
    lines = [var.split("=") for var in env_content.split("\n") if var]
    key_val_list = [f'$env:{key.strip()}="{val.strip()}"' for key, val in lines]
    pwsh_str = " ; ".join(key_val_list)
    pyperclip.copy(pwsh_str)
    return pwsh_str


def create_cmd_string(env_content: str):
    lines = [var.split("=") for var in env_content.split("\n") if var]
    key_val_list = [f'set "{key.strip()}={val.strip()}"' for key, val in lines]
    cmd_str = " & ".join(key_val_list)
    pyperclip.copy(cmd_str)
    return cmd_str


def create_bash_string(env_content: str):
    lines = [var.split("=") for var in env_content.split("\n") if var]
    key_val_list = [f"export {key.strip()}={val.strip()}" for key, val in lines]
    bash_str = " ; ".join(key_val_list)
    pyperclip.copy(bash_str)
    return bash_str


def write_to_file(path: Path, content):
    path.parent.mkdir(parents=True, exist_ok=True)

    with open(path, "w") as env_file:
        env_file.write(content)



from pathlib import Path

from textual import on
from textual.containers import Container, Vertical
from textual.widgets import Button, Input, Label, ListView, TextArea

from ..utils import (
    copy_path_to_clipboard,
    create_copy_in_cwd,
    create_shell_export_str,
    write_to_file,
)
from .modals import ModalSaveScreen, ModalShellSelector


class InteractionPanel(Container):
    def compose(self):
        yield Button(
            "Create Shell String",
            id="btn-shell-export",
            disabled=True,
            variant="primary",
        )
        yield Button(
            "Export File to current dir",
            id="btn-file-export",
            disabled=True,
            variant="primary",
        )
        yield Button(
            "Copy Path to Clipboard",
            id="btn-copy-path",
            disabled=True,
            variant="primary",
        )
        with Vertical(id="interaction-shell-select"):
            yield Label("Select your Shell")
            yield Button(
                label=self.app.current_shell, id="btn-shell-select", variant="primary"
            )
        with Vertical(id="interaction-export-name"):
            yield Label("Export filename")
            yield Input(
                value=".env",
                placeholder="env file name for export",
                id="export-env-name",
            )
        yield Button(
            "New Env File", id="btn-new-file", disabled=False, variant="success"
        )
        yield Button(
            "Save Env File", id="btn-save-file", disabled=True, variant="success"
        )

    # Export Interactions
    @on(Button.Pressed, "#btn-copy-path")
    def copy_env_path(self):
        copy_path_to_clipboard(path=self.app.file_to_show_path)

    @on(Button.Pressed, "#btn-file-export")
    def export_env_file(self):
        export_filename = self.query_one(Input).value
        create_copy_in_cwd(
            filename=export_filename, filepath=self.app.file_to_show_path
        )

    @on(Button.Pressed, "#btn-shell-export")
    def export_env_str_shell(self):
        create_shell_export_str(
            shell=self.app.current_shell, env_content=self.app.current_content
        )

    # Shell Select Interactions
    @on(Button.Pressed, "#btn-shell-select")
    def pop_modal_shell(self):
        self.app.push_screen(ModalShellSelector())

    # Env File Interactions
    @on(Button.Pressed, "#btn-new-file")
    def new_file(self, event: Button.Pressed):
        self.app.file_to_show = ""
        self.app.file_to_show_path = ""
        self.app.current_content = ""

        text_widget = self.app.query_one(TextArea)
        text_widget.text = ""
        text_widget.disabled = False
        text_widget.focus()

        event.button.disabled = True
        self.query_one("#btn-save-file").disabled = False

        self.app.query_one("#file-preview").border_title = "Creating New .Env File ..."

        for views in self.app.query(ListView):
            views.query(Button).remove_class("active")
            views.index = None

    @on(Button.Pressed, "#btn-save-file")
    def save_file(self, event: Button.Pressed):
        text_widget = self.app.query_one(TextArea)
        self.app.current_content = text_widget.text
        text_widget.disabled = True
        self.query_one("#btn-new-file").disabled = False
        event.button.disabled = True

        if self.app.file_to_show:
            write_to_file(
                path=Path(self.app.file_to_show_path), content=self.app.current_content
            )
        else:
            self.app.push_screen(ModalSaveScreen())



import configparser

from .constants import (
    CONFIG_FILE_NAME,
    CONFIG_FILE_PATH,
    CONFIG_PATH,
    DATA_PATH,
    ENV_FILE_DIR_PATH,
)


def create_init_config(conf_path=CONFIG_PATH, data_path=DATA_PATH):
    config = configparser.ConfigParser(default_section=None)
    config.optionxform = str
    config["settings"] = {"Shell": "bash"}

    if not ENV_FILE_DIR_PATH.exists():
        data_path.mkdir(exist_ok=True)
        ENV_FILE_DIR_PATH.mkdir(exist_ok=True)

    with open(conf_path / CONFIG_FILE_NAME, "w") as configfile:
        config.write(configfile)


def check_config_exists(path=CONFIG_FILE_PATH):
    return path.exists()


class DotEnvHubConfig:
    def __init__(self, path=CONFIG_FILE_PATH) -> None:
        self.configpath = path
        self._config = configparser.ConfigParser(default_section=None)
        self._config.optionxform = str
        self._config.read(path)

    def save(self):
        with open(self.configpath, "w") as configfile:
            self.config.write(configfile)

    @property
    def config(self) -> configparser.ConfigParser:
        return self._config

    @property
    def shell(self) -> str:
        return self.config["settings"]["Shell"]

    @shell.setter
    def shell(self, new_shell) -> str:
        self._config["settings"]["Shell"] = new_shell
        self.save()


if not check_config_exists():
    create_init_config()

cfg = DotEnvHubConfig()



from pathlib import Path

from platformdirs import user_config_dir, user_data_dir

ENV_FILE_DIR_NAME = "env_files"
CONFIG_FILE_NAME = "dotenvhub.ini"

CONFIG_PATH = Path(
    user_config_dir(appname="dotenvhub", appauthor=False, ensure_exists=True)
)
DATA_PATH = Path(
    user_data_dir(appname="dotenvhub", appauthor=False, ensure_exists=True)
)

CONFIG_FILE_PATH = CONFIG_PATH / CONFIG_FILE_NAME
ENV_FILE_DIR_PATH = DATA_PATH / ENV_FILE_DIR_NAME

SHELLS = [
    "pwsh",
    "cmd",
    "bash",
    "zsh",
]



"""
This is a skeleton file that can serve as a starting point for a Python
console script. To run this script uncomment the following lines in the
``[options.entry_points]`` section in ``setup.cfg``::

    console_scripts =
         fibonacci = dotenvhub.skeleton:run

Then run ``pip install .`` (or ``pip install -e .`` for editable mode)
which will install the command ``fibonacci`` inside your current environment.

Besides console scripts, the header (i.e. until ``_logger``...) of this file can
also be used as template for Python modules.

Note:
    This file can be renamed depending on your needs or safely removed if not needed.

References:
    - https://setuptools.pypa.io/en/latest/userguide/entry_point.html
    - https://pip.pypa.io/en/stable/reference/pip_install
"""

import sys

from dotenvhub import cli_parser, constants, tui, utils

__author__ = "Zaloog"
__copyright__ = "Zaloog"
__license__ = "MIT"


# ---- Python API ----
# The functions defined in this section can be imported by users in their
# Python scripts/interactive interpreter, e.g. via
# `from dotenvhub.skeleton import fib`,
# when using this Python module as a library.


# ---- CLI ----
# The functions defined in this section are wrappers around the main Python
# API allowing them to be called directly from the terminal as a CLI
# executable/script.


def main(args):
    """Wrapper allowing :func:`fib` to be called with string arguments in a CLI fashion

    Instead of returning the value from :func:`fib`, it prints the result to the
    ``stdout`` in a nicely formatted message.

    Args:
      args (List[str]): command line parameters as list of strings
          (for example  ``["--verbose", "42"]``).
    """
    parsed_args = cli_parser.parse_args(args)

    if not args:
        tui.DotEnvHub().run()

    if parsed_args.mode == "shell":
        content = utils.get_env_content(
            filepath=constants.ENV_FILE_DIR_PATH / parsed_args.filename
        )

        shell_str = utils.create_shell_export_str(
            shell=parsed_args.shell, env_content=content
        )

        utils.console.print(f"Copied [blue]{shell_str}[/] to clipboard")

    if parsed_args.mode == "copy":
        utils.create_copy_in_cwd(
            filepath=constants.ENV_FILE_DIR_PATH / parsed_args.filename,
            filename=parsed_args.export_name,
        )


def run():
    """Calls :func:`main` passing the CLI arguments extracted from :obj:`sys.argv`

    This function can be used as entry point to create console scripts with setuptools.
    """
    main(sys.argv[1:])


if __name__ == "__main__":
    # ^  This is a guard statement that will prevent the following code from
    #    being executed in the case someone imports this file instead of
    #    executing it as a script.
    #    https://docs.python.org/3/library/__main__.html

    # After installing your project with pip, users can also run your Python
    # modules as scripts via the ``-m`` flag, as defined in PEP 338::
    #
    #     python -m dotenvhub.skeleton 42
    #
    run()



from pathlib import Path

from textual.app import App, ComposeResult
from textual.containers import Container
from textual.reactive import var
from textual.widgets import Footer, Header

from .config import cfg
from .utils import update_file_tree
from .widgets.filepanel import EnvFileSelector
from .widgets.interactionpanel import InteractionPanel
from .widgets.previewpanel import FilePreviewer


class DotEnvHub(App):
    CSS_PATH = Path("assets/tui.css")

    file_to_show = var("")
    file_to_show_path = var("")
    file_tree = var(update_file_tree())
    current_content = var("")
    current_shell = var(cfg.shell)

    def compose(self) -> ComposeResult:
        yield Header()
        yield Footer()

        with Container(id="app-grid"):
            file_selector = EnvFileSelector(id="file-selector")
            file_selector.border_title = "Select your .env File"
            yield file_selector

            file_previewer = FilePreviewer(id="file-preview")
            file_previewer.border_title = "No Env File Selected"
            yield file_previewer

            file_interaction = InteractionPanel(id="interaction")
            file_interaction.border_title = "What do you want to do?"
            yield file_interaction
